/**
 * 
 */
package parte04;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.AbstractTableModel;

/**
 * @author vitor
 *
 */
public class JanelaConsulta extends JFrame {
	
	public JanelaConsulta() {
		super("Consultas SQL");
		initComponents();
	}
	
	/**
	 * 
	 */
	private void initComponents() {
		// Configura os elementos gráficos.
		setLayout(new BorderLayout(10, 10));
		
		JPanel painelTopo = new JPanel(new BorderLayout(5, 5));
		
		JLabel rotuloSQL = new JLabel("SQL:");
		painelTopo.add(rotuloSQL, BorderLayout.LINE_START);
		
		final JTextArea textoSQL = new JTextArea();
		textoSQL.setRows(5);
		painelTopo.add(textoSQL, BorderLayout.CENTER);
		
		JPanel painelBotoes = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 5));
		JButton botaoSQL = new JButton("Enviar");
		painelBotoes.add(botaoSQL);
		painelTopo.add(painelBotoes, BorderLayout.PAGE_END);
		
		add(painelTopo, BorderLayout.PAGE_START);
		
		final JTable tabelaResultados = new JTable();
		add(new JScrollPane(tabelaResultados), BorderLayout.CENTER);
		
		// Configura a própria janela.
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Adiciona ouvinte para o botão.
		botaoSQL.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tabelaResultados.setModel(new ResultSetTableModel(textoSQL.getText()));
			}
		});
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		JanelaConsulta janela = new JanelaConsulta();
		janela.setSize(640, 480);
		janela.setVisible(true);
	}

}

class ResultSetTableModel extends AbstractTableModel {
	private ResultSet rset;
	private ResultSetMetaData meta;
	private int rows;
	
	public ResultSetTableModel(String sql) {
		try {
			// Carrega o driver do HSQLDB e conecta.
			Class.forName(Config.NOME_DRIVER);
			Connection conn = DriverManager.getConnection(Config.BD_URL, Config.BD_LOGIN, Config.BD_SENHA);		
			
			// Cria o statement navegável em qualquer direção.
			Statement stmt = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			
			// Executa a consulta e obtém os meta-dados.
			rset = stmt.executeQuery(sql);
			meta = rset.getMetaData();
			
			// Determina o número de linhas.
			rset.last();
			rows = rset.getRow();
			
			// Notifica alteração no modelo.
			fireTableStructureChanged();
		}
		catch (ClassNotFoundException e) {
			JOptionPane.showMessageDialog(null, "Driver não encontrado!", "Erro de Driver", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		catch (SQLException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Erro SQL", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
	}
	
	public Class getColumnClass(int column) {
		String className = null;
		try {
			className = meta.getColumnClassName(column + 1);
			return Class.forName(className);
		}
		catch (SQLException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Erro SQL", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		catch (ClassNotFoundException e) {
			JOptionPane.showMessageDialog(null, "Classe não encontrada: " + className, "Erro de Carregamento de Classe", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		return Object.class;
	}
	
	public int getColumnCount() {
		try {
			return meta.getColumnCount();
		}
		catch (SQLException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Erro SQL", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		return 0;
	}
	
	public String getColumnName(int column) {
		try {
			return meta.getColumnName(column + 1);
		}
		catch (SQLException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Erro SQL", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		return "";
	}
	
	public int getRowCount() {
		return rows;
	}
	
	public Object getValueAt(int row, int column) {
		try {
			rset.absolute(row + 1);
			return rset.getObject(column + 1);
		}
		catch (SQLException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Erro SQL", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		return "";
	}
}