/**
 * 
 */
package parte04.persistencia;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import parte04.dominio.Artista;

public class ArtistaDAOJDBC extends DAOBaseJDBC implements ArtistaDAO {
	/* (non-Javadoc)
	 * @see parte04.persistencia.ArtistaDAO#excluir(parte04.dominio.Artista)
	 */
	public void excluir(Artista artista) {
		try {
			if (artista.getId() != null) {
				PreparedStatement stmt = conn.prepareStatement("DELETE FROM Artista WHERE id = ?");
				stmt.setLong(1, artista.getId());
				stmt.executeUpdate();
			}
		}
		catch (SQLException e) {
			System.out.println("Erro SQL: " + e.getMessage());
			System.exit(1);
		}
	}

	/* (non-Javadoc)
	 * @see parte04.persistencia.ArtistaDAO#obterPorPais(java.lang.String)
	 */
	public Collection obterPorPais(String pais) {
		Set set = new HashSet();
		try {
			PreparedStatement stmt = conn.prepareStatement("SELECT id, nome, banda, pais FROM Artista WHERE pais LIKE ?");
			stmt.setString(1, "%" + pais + "%");
			preencheConjuntoArtista(set, stmt.executeQuery());
		}
		catch (SQLException e) {
			System.out.println("Erro SQL: " + e.getMessage());
			System.exit(1);
		}
		return set;
	}

	/* (non-Javadoc)
	 * @see parte04.persistencia.ArtistaDAO#obterTodos()
	 */
	public Collection obterTodos() {
		Set set = new HashSet();
		try {
			PreparedStatement stmt = conn.prepareStatement("SELECT id, nome, banda, pais FROM Artista");
			preencheConjuntoArtista(set, stmt.executeQuery());
		}
		catch (SQLException e) {
			System.out.println("Erro SQL: " + e.getMessage());
			System.exit(1);
		}
		return set;
	}

	/* (non-Javadoc)
	 * @see parte04.persistencia.ArtistaDAO#recuperar(java.lang.Long)
	 */
	public Artista recuperar(Long id) {
		Artista artista = null;
		try {
			PreparedStatement stmt = conn.prepareStatement("SELECT id, nome, banda, pais FROM Artista WHERE id = ?");
			stmt.setLong(1, id.longValue());
			ResultSet rset = stmt.executeQuery();
			if (rset.next()) {
				artista = new Artista();
				artista.setId(new Long(rset.getLong("id")));
				artista.setNome(rset.getString("nome"));
				artista.setBanda(new Boolean(rset.getBoolean("banda")));
				artista.setPais(rset.getString("pais"));				
			}
		}
		catch (SQLException e) {
			System.out.println("Erro SQL: " + e.getMessage());
			System.exit(1);
		}
		return artista;
	}

	/* (non-Javadoc)
	 * @see parte04.persistencia.ArtistaDAO#salvar(parte04.dominio.Artista)
	 */
	public void salvar(Artista artista) {
		PreparedStatement stmt;

		try {
			if (artista.getId() == null) stmt = conn.prepareStatement("INSERT INTO Artista (nome, banda, pais) VALUES (?, ?, ?)");
			else {
				stmt = conn.prepareStatement("UPDATE Artista SET nome = ?, banda = ?, pais = ? WHERE id = ?");
				stmt.setLong(4, artista.getId());
			}
			
			stmt.setString(1, artista.getNome());
			stmt.setBoolean(2, artista.getBanda().booleanValue());
			stmt.setString(3, artista.getPais());
			stmt.executeUpdate();
		}
		catch (SQLException e) {
			System.out.println("Erro SQL: " + e.getMessage());
			System.exit(1);
		}
	}

	/**
	 * @param set
	 * @param rset
	 */
	private void preencheConjuntoArtista(Set set, ResultSet rset) throws SQLException {
		while (rset.next()) {
			Artista artista = new Artista();
			artista.setId(new Long(rset.getLong("id")));
			artista.setNome(rset.getString("nome"));
			artista.setBanda(new Boolean(rset.getBoolean("banda")));
			artista.setPais(rset.getString("pais"));
			set.add(artista);
		}
	}
}
